/************************************************************************/
/* header file for mathc library 2.2					*/
/* 03.01.2003								*/
/************************************************************************/


/* numerical constants */
#define NMAX		16		/* max. matrix size */
#define M_PI            3.14159265358979323846 /* Pi from math.h */
#define DHIGH		1.0E+99
#define DEPS		1.0E-9		/* best rel. precision */

/* macros */
#define SQ(X)		((X)*(X))       /* square */
#define CB(X)		((X)*(X)*(X))   /* cube */
#ifndef MAX
  #define MAX(X,Y)	(((X)>(Y))? (X) : (Y))
#endif
#ifndef MIN
  #define MIN(X,Y)	(((X)<(Y))? (X) : (Y))
#endif
#ifndef SWAP
  #define SWAP(A,B,AUX)	{ AUX = A; A = B; B = AUX; }
#endif


/* structures */

typedef struct {			/* double matrix */
  double **a;
  double **row;				/* for array of row pointers */
  double *data;				/* data space for elements */
  int nrl, nrh;				/* row index range */
  int ncl, nch;				/* column index range */
  int nrow, ncol;			/* number of rows and columns */
  int n;				/* number of elements */
} dmatrix;

typedef struct {			/* integer matrix */
  int **a;
  int **row;				/* for array of row pointers */
  int *data;				/* data space for elements */
  int nrl, nrh;				/* row index range */
  int ncl, nch;				/* column index range */
  int nrow, ncol;			/* number of rows and columns */
  int n;				/* number of elements */
} imatrix;

typedef struct {			/* double vector */
  double *a;
  double *data;				/* data space for elements */
  int nl, nh;				/* index range */
  int n;				/* number of elements */
} dvector;

typedef struct {			/* integer vector */
  int *a;
  int *data;				/* data space for elements */
  int nl, nh;				/* index range */
  int n;				/* number of elements */
} ivector;

typedef struct {			/* point in a plane */
  double x, y;
} point2;


/* prototypes of functions in mathc library */

int eqn_quadratic(const double *a, double *x);

int eqn_cubic(const double *a, double *x);

int eqn_quartic(const double *a, double *x);

int eqn_quintic(const double *a, double *x);

double regula_falsi(double (*f)(), void *s, double xa, double xb,
double ya, double yb, double eps);

int eqn_steffensen(double (*f)(), void *u, double *x, int kmax, double eps);

int diff_ridders(double (*f)(), void *aux, int n, double x, double h,
double *fd, double *err);

int diff_romberg(double (*f)(), void *aux, int n, double x, double h,
int maxord, double xmin, double xmax, double *fd);

double diff_x(double (*f)(), void *aux, double x, double dx);

int diff_2all(double (*f)(), double x, double y, void *s, int n, int r,
double dx, double dy, double *d);

int fit_marquardt(int (*f)(), int m, double *x, int n, void *aux,
int maxit, int iprot, double *ynorm);

int fit_powell(int (*obj)(), int m, double *x, int n, void *aux,
int maxit, int iprot, double *rms);
int min_powell(int (*obj)(), int m, double *x, void *aux,
int maxit, int iprot, double *rms);

double int_gauss(double (*f)(), void *u, int k, double a, double b, int n);

int int_romberg(double (*f)(), void *u, double a, double b, double eps,
int maxord, double *q);

double int_simpson(double (*f)(), void *u, double a, double b, int n);

int ipl_akima(int n, double *x, double *y, double *b, double *c,
double *d);

int ipl_pspline(int n, double *x, double *y, double *b, double *c,
double *d);
int ipl_spline(int n, double *x, double *y, double *b, double *c,
double *d);

double ipl_eval(double xvalue, int n, double *x, double *y,
double *b, double *c, double *d);

double ipl_peval(double xvalue, int n, double *x, double *y,
double *b, double *c, double *d);

int ipl_lagrange(int n, double *x, double *y, double *a);

void track(double x, int n, double *y, int mode);

int linsys_gauss(const dmatrix *a, const dvector *b, dvector *x);

int min_brent(int (*f)(), void *aux, double xa, double xb, double xc,
double eps, double *xmin, double *ymin);
double min_parabola(point2 a, point2 b, point2 c);

int minimum1(double (*f)(), void *s, double a, double b, double step,
double eps, double *xmin, double *ymin);

double ran_cgr48(void);

double ran_ecuyer(void);

double ran_stauffer(void);

void ran_vector(double (*random)(), int n, double *u);

/* array_ops.c */
void vadd(int n, double *v1, double *v2, double *v3);
void vsub(int n, double *v1, double *v2, double *v3);
void vmult(int n, double *v1, double *v2, double *v3);
void vdiv(int n, double *v1, double *v2, double *v3);
void vsmult(int n, double *v1, double *v3, double a);
void vsadd(int n, double *v1, double *v3, double a);
void vset(int n, double *v3, double a);
void vcopy(int n, double *v1, double *v3);
void vfunc(int n, double *v1, double *v3, double (*f)());
double dot(int n, double *a, double *b);

/* matrix_ops.c */
int dmatrix_mul(const dmatrix *a, const dmatrix *b, dmatrix *c);
int dmatrix_mult(const dmatrix *a, const dmatrix *b, dmatrix *c);
int dmatrix_mulv(const dmatrix *a, const dvector *b, dvector *c);
void dmatrix_set(double x, dmatrix *b);
int dmatrix_copy(const dmatrix *a, dmatrix *b);
double dmatrix_dot(const dmatrix *a, const dmatrix *b);
int dmatrix_invert(dmatrix *a);

/* vector_ops.c */
int dvector_add(const dvector *v1, const dvector *v2, dvector *v3);
int dvector_sub(const dvector *v1, const dvector *v2, dvector *v3);
int dvector_mul(const dvector *v1, const dvector *v2, dvector *v3);
int dvector_div(const dvector *v1, const dvector *v2, dvector *v3);
int dvector_smul(const dvector *v1, double a, dvector *v3);
int dvector_sadd(const dvector *v1, double a, dvector *v3);
int dvector_set(double a, dvector *v3);
int dvector_copy(const dvector *v1, dvector *v3);
int dvector_func(double (*f)(), const dvector *v1, dvector *v3);
double dvector_dot(const dvector *v1, const dvector *v2);

/* matrix_dynamic.c */
int dmatrix_create(dmatrix *m, int nrl, int nrh, int ncl, int nch);
void dmatrix_free(dmatrix *m);
int imatrix_create(imatrix *m, int nrl, int nrh, int ncl, int nch);
void imatrix_free(imatrix *m);
int dvector_create(dvector *v, int nl, int nh);
void dvector_free(dvector *v);
int ivector_create(ivector *v, int nl, int nh);
void ivector_free(ivector *v);

/* matrix_fixed.c */
void dmatrix_overlay(dmatrix *m, double **p, double *a, int nrl, int nrh,
int ncl, int nch);
void imatrix_overlay(imatrix *m, int **p, int *a, int nrl, int nrh,
int ncl, int nch);
void dvector_overlay(dvector *m, double *a, int nl, int nh);
void ivector_overlay(ivector *m, int *a, int nl, int nh);

/* old.c: old versions (kept for compatibility) */
int akima(int n, double *x, double *y, double *b, double *c, double *d);
int cubic(const double *a, double *x);
int deriv1(double (*f)(), int n, double x, double h, double *fd,
double *err);
int deriv2all(double (*f)(), double x, double y, void *s, int n, int r,
double dx, double dy, double *d);
double intgauss(double (*f)(), int k, double a, double b, int n);
int intromberg(double (*f)(), double a, double b, double eps,
int maxord, double *q);
double intsimpson(double (*f)(), double a, double b, int n);
int marqua(int (*f)(), int m, double *x, int n, void *aux,
int maxit, int iprot, double *ynorm);
int minimum(double (*f)(), void *s, double a, double b, double step,
double eps, double *xmin, double *ymin);
int powell(int (*obj)(), int m, double *x, int n, void *aux,
int maxit, int iprot, double *rms);
int quadratic(const double *a, double *x);
int quartic(const double *a, double *x);
int quintic(const double *a, double *x);
double ran2(void);
double ran2_ecu(void);
double ran_st(void);
void ranvector(double (*random)(), int n, double *u);
double regfalsi(double (*f)(), void *s, double xa, double xb,
double ya, double yb, double eps);
int spline(int n, double *x, double *y, double *b, double *c,
double *d);
double spval(double xvalue, int n, double *x, double *y,
double *b, double *c, double *d);
int steff1(double (*f)(), double *x, int kmax, double eps);

/* old_matrix.c */
int linsys(int n, double *a, double *b, double *x);
int matinv(int n, double *a);
void mpro(int l, int m, double *a, int n, double *b, double *c);
void mprot(int l, int m, double *a, int n, double *b, double *c);
int create_dmatrix(double ***m, int nrl, int nrh, int ncl, int nch);
int create_imatrix(int ***m, int nrl, int nrh, int ncl, int nch);
int create_dvector(double **v, int nl, int nh);
int create_ivector(int **v, int nl, int nh);
void free_dmatrix(double **m, int nrl, int ncl);
void free_imatrix(int **m, int nrl, int ncl);
void free_dvector(double *v, int nl);
void free_ivector(int *v, int nl);
double **overlay_dmatrix(double **p, double *a, int nrl, int nrh, int ncl,
int nch);
int **overlay_imatrix(int **p, int *a, int nrl, int nrh, int ncl, int nch);
