/*************************************************************10.12.2001*/
/* int_simpson: Integration of a function by means of Simpson's method	*/	
/*									*/
/* If the number of panels is not even (a requirement for Simpson's	*/
/* method, the last three panels are treated by Newton's 3/8 method.	*/
/************************************************************************/
/* f(x, u)	(i)  integrand function (x: scalar)
   u		(i)  structure for passing additional information
   a, b		(i)  interval limits
   n		(i)  number of panels
   RESULT	(o)  integral value					*/
/*======================================================================*/
#include <math.h>
#include <mathc.h>

double int_simpson(double (*f)(), void *u, double a, double b, int n) {
  int i, nh;
  double h, sum_odd, sum_even, sum, yh;

  h = (b - a)/n;			/* panel width */
  if (n <= 0)
    sum = 0.0;
  else if (n == 1)
    sum = ((*f)(a, u) + (*f)(b, u)) *h;
  else {
    nh = (n % 2 == 0) ? n : n - 3;	/* number of Simpson panels */
    yh = (*f)(a + nh * h, u);
    sum = sum_odd = sum_even = 0.0;
    if (nh > 0) {
      for (i = 1; i <= nh - 1; i += 2) sum_odd += (*f)(a + i*h, u);
      for (i = 2; i <= nh - 2; i += 2) sum_even += (*f)(a + i*h, u);
      sum += ((*f)(a, u) + 4 * sum_odd + 2 * sum_even + yh) * h/3.0;
    }
    if (n != nh) sum += (yh + 3 * (*f)(a + (nh+1) * h, u)
    + 3 * (*f)(a + (nh+2) * h, u) + (*f)(b, u)) * h * 0.375;
  }
  return (sum);
}
