/*************************************************************02.01.2003*/
/* ipl_lagrange: Lagrange interpolation polynomial			*/
/*									*/
/* G. Jordan-Engeln, F. Reutter: Numerische Mathematik fr Ingenieure,	*/
/* BI Hochschultaschenbcher Bd. 104, Bibliographisches Institut,	*/
/* Mannheim 1973, pp. 198ff						*/
/************************************************************************/
/* n		(i)  number of data points
   x[n], y[n]	(i)  abscissa and ordinate values
   a[n]		(o)  coefficients of Lagrange polynomial
   RESULT	(o)  error code: 0 = no error
				 1 = some x values are identical
				 2 = could not allocate memory		*/
/*======================================================================*/
#include <math.h>
#include <mathc.h>
#include <stdlib.h>

int ipl_lagrange(int n, double *x, double *y, double *a) {
  int i, j, k, err;
  double *poly, h1, h2, f;

  if ((poly = (double *) malloc((size_t) (n * sizeof(double)))) == NULL)
  return (2);

  err = 0;
  for (i = 0; i < n; i++) a[i] = 0.0;

  for (k = 0; k < n; k++) {
    poly[0] = 1.0;
    for (i = 1; i < n; i++) poly[i] = 0.0;
    for (j = 0, f = 1.0; j < n; j++) {
      if (j == k) continue;
      for (i = 0, h1 = 0.0; i < n-1; i++) {
	h2 = poly[i];
	poly[i] = -x[j] * poly[i] + h1;
	h1 = h2;
      }
      poly[n-1] = h1;
      f *= (x[k] - x[j]);
    }
    if (f == 0.0) {
      err = 1;
      break;
    }
    for (i = 0; i < n; i++) a[i] += y[k]/f * poly[i];
  }
  free(poly);
  return (err);
}
