/*************************************************************26.12.2001*/
/* min_brent: minimum of a 1-dimensional function by Brent's method	*/
/*									*/
/* NOTE: Convergence is guaranteed if the ordinate of the middle init-	*/
/*	 ial value is lower than the ordinate values of the others.	*/
/*	 If more than one local minimum exists in the range defined by	*/
/*	 the initial values, the minimum found may not be the deepest	*/
/*	 one.								*/
/************************************************************************/
/* f(x, y, aux)		(i)  function to be minimized
   aux			(i)  structure passing additional data to f()
   xa, xb, xc		(i)  initial abscissa values (can be in any order)
   eps			(i)  desired absolute x accuracy
   xmin, ymin		(o)  minimum coordinates
   RESULT		(o)  error code: 0 = no error
					 1 = no minimum
				  10*k + 2 = error #k of f()		*/
/*======================================================================*/
#include <math.h>
#include <mathc.h>

#define	GOLD		1.618034	/* sectio aurea ratio */
#define CGOLD		(2.0 - GOLD)

int min_brent(int (*f)(), void *aux, double xa, double xb, double xc,
double eps, double *xmin, double *ymin) {
  int err;
  double e, e_old, d;
  point2 a, b, c, p;

  if (xa > xb) SWAP(xa, xb, e);		/* sort initial abscissa values */
  if (xb > xc) SWAP(xb, xc, e);
  if (xa > xb) SWAP(xa, xb, e);
  
  a.x = xa;				/* calc. and check ordinates */
  if ((err = (*f)(a.x, &a.y, aux)) != 0) return (10*err + 2);
  b.x = xb;
  if ((err = (*f)(b.x, &b.y, aux)) != 0) return (10*err + 2);
  c.x = xc;
  if ((err = (*f)(c.x, &c.y, aux)) != 0) return (10*err + 2);
  if (b.y > a.y || b.y > c.y) return (1);

  e = d = DHIGH;
  while (c.x - a.x > eps) {
    p.x = min_parabola(a, b, c);	/* trial parabolic extrapol. */
    e_old = e;
    e = d;
    d = p.x - b.x;
    if (p.x <= a.x || p.x >= c.x || fabs(d) > 0.5*fabs(e_old)) {
      e = ((b.x >= 0.5*(a.x + c.x))? a.x : c.x) - b.x;
      d = CGOLD*e;			/* sectio aurea */
      p.x = b.x + d;
    }
    if ((err = (*f)(p.x, &p.y, aux)) != 0) return (10*err + 2);
    if (p.y <= b.y) {			/* p lower than previous best? */
      if (p.x >= b.x)
	a = b;
      else
	c = b;
      b = p;
    }
    else {
      if (p.x >= b.x)
	c = p;
      else
	a = p;
    }
  }
  *xmin = b.x;
  *ymin = b.y;
  return (0);
}

/*************************************************************21.12.2001*/
/* min_parabola: abscissa of the minimum of a parabola through 3 points	*/
/************************************************************************/
/* a, b, c	(i)  points
   RESULT	(o)  abscissa value at parabola minimum; if the points
		     are collinear, a very high value is returned.	*/
/*======================================================================*/
double min_parabola(point2 a, point2 b, point2 c) {
  double r, q;

  r = (b.x - a.x) * (b.y - c.y);
  q = (b.x - c.x) * (b.y - a.y);
  if (fabs(q - r) < DEPS * fabs(q + r))
    return ((q > r)? DHIGH : -DHIGH);
  else
    return (b.x- ((b.x - c.x)*q - (b.x - a.x)*r)/(2*(q - r)));
}
