/*************************************************************24.08.2001*/
/* minimum1: find the global minimum of a 1-dimensional function	*/
/*									*/
/* An interval is scanned with a step size that is passed as a para-	*/
/* meter. If a minimum is detected, i.e. if an y-value is found that is	*/
/* lower than the y-values at adjacent positions, this minimum is	*/
/* located by iterative parabola fitting.				*/
/************************************************************************/
/* f		(i)  function fo which the minumum is sought
   s		(i)  structure for passing additional data to f()
   a, b		(i)  interval for the search (a < b!)
   step		(i)  step size of initial search
   eps		(i)  desired accuracy of xmin
   xmin, ymin	(o)  abscissa and ordinate of minimum
   RESULT	(o)  error code: 0 = no error, minimum detected
				 1 = no relative minimum; on eof the
				     interval boundaries is returned
				 2 = search went astray
				     (step too large?)			*/
/*======================================================================*/
#include <math.h>
#include <mathc.h>

int minimum1(double (*f)(), void *s, double a, double b, double step,
double eps, double *xmin, double *ymin) {
  int err, localmin;
  double dx, det, a1, a2, xres;
  struct point {
    double x, y;
  } p0, p1, p2;

  p0.x = *xmin = a;			/* start scan at left boundary */
  p0.y = *ymin = (*f)(a, s);
  err = 1;
  if ((p1.x = a + step) >= b) return (2);
  p1.y = (*f)(p1.x, s);

  while (p1.x < b) {			/* main scanning loop */
    p2.x = MIN(p1.x + step, b);
    p2.y = (*f)(p2.x, s);

    do {				/* construct parabola */
      localmin = 0;
      det = (p2.x - p1.x) * (p0.x - p1.x) * (p2.x - p0.x);
      a2 = ((p2.y - p1.y) * (p0.x - p1.x) - (p0.y - p1.y)
      * (p2.x - p1.x))/det;
      a1 = ((p0.y - p1.y) * SQ(p2.x - p1.x) - (p2.y - p1.y)
      * SQ(p0.x - p1.x))/det;
      if (a2 <= 0.0) break;		/* wrong curvature? */

      dx = -0.5*a1/a2;			/* calculate the parabola minimum */
      xres = p1.x + dx;
      if (xres <= p0.x)
	break;
      else if (xres < p1.x)
	p2 = p1;
      else if (xres < p2.x)
	p0 = p1;
      else
	break;
      localmin = 1;
      p1.x = xres;
      p1.y = (*f)(xres, s);
    } while (fabs(dx) > eps);

    if (localmin) {
      if (p1.y < *ymin) {		/* compare with previous minimum */
	*xmin = p1.x;
	*ymin = p1.y;
	err = 0;
      }
      p0 = p1;				/* resume scanning */
      p1.x = MIN(p0.x + step, b);
      p1.y = (*f)(p1.x, s);
    }
    else {
      p0 = p1;
      p1 = p2;
    }
  }

  if (p1.y < *ymin) {			/* minimum on interval boundary? */
    *xmin = p1.x;
    *ymin = p1.y;
    err = 1;
  }
  return (err);
}
