/*************************************************************21.12.2001*/
/* regula_falsi: fast 1-dimensional regula falsi iteration (Illinois	*/
/*		 algorithm)						*/
/*									*/
/* The input parameters are not changed. Convergence is certain, if	*/
/* ya and yb have opposite signs.					*/
/*									*/
/* NOTE: The signs of ya, yb, or eps are not tested by this function.	*/
/************************************************************************/
/* f(x, s)	(i)  object function
   s		(i)  structure passing additional data to f()
   xa, xb	(i)  initial values
   ya, yb	(i)  residuals at xa and xb (ya = f(xa), yb = f(xb))
   eps		(i)  desired precision
   RESULT	(o)  x value for which f(x) = 0				*/
/*======================================================================*/
#include <math.h>
#include <mathc.h>

double regula_falsi(double (*f)(), void *s, double xa, double xb, double ya,
double yb, double eps) {
  double dx, h;
  point2 p1, p2, pnew;

  p1.x = xa;
  p1.y = ya;
  p2.x = xb;
  p2.y = yb;
  while (1) {			/* loop terminated from within! */
    dx = p1.y * (p1.x - p2.x)/(p1.y - p2.y);
    pnew.x = p1.x - dx;
    if (fabs(dx) < eps) break;
    pnew.y = (*f)(pnew.x, s);
    h = pnew.y * p1.y;
    if (h < 0.0)
      p2 = p1;
    else if (h == 0.0)
      break;
    else
      p2.y *= 0.5;
    p1 = pnew;
  }
  return (pnew.x);
}
