/*************************************************************10.10.1994*/
/* track: generate initial values for iterative procedures by		*/
/*	  extrapolation from previous results				*/
/*									*/
/* Track stores up to VMAX previous vectors. If more vectors are stored,*/
/* the oldest entry is overwritten.					*/
/************************************************************************/
/* x		(i)  independent scan parameter 
   n		(i)  number of initial values (must be < NMAX)
   y[n]		(io) vector of initial values
   mode		(i)  mode control: 0 = reset
				   1 = store a set of initial values
				   2 = extrapolate			*/
/*======================================================================*/
#include <math.h>
#include <mathc.h>

#define VMAX		3		/* number of entries stored */

void track(double x, int n, double *y, int mode) {
  int i, j;
  double h, d12, d13, d23;
  static int k;
  static double xtbl[VMAX], ytbl[VMAX][NMAX];

  switch(mode) {
  case 0:				/* purge table */
    k = 0;
    break;
  case 1:				/* enter a new vector */
    if (k == VMAX) {
      for (i = 1; i < VMAX; i++) {
	xtbl[i-1] = xtbl[i];
	for (j = 0; j < n; j++) ytbl[i-1][j] = ytbl[i][j];
      }
    }
    if (k < VMAX) k++;
    xtbl[k-1] = x;
    for (j = 0; j < n; j++) ytbl[k-1][j] = y[j];
    break;
  case 2:				/* extrapolate */
    if (k >= 1 ) {
      for (j = 0; j < n; j++) y[j] = ytbl[k-1][j];
    }
    if (k >= 2) {			/* ... linear */
      d12 = xtbl[k-1] - xtbl[k-2];
      h = (x - xtbl[k-1])/d12;
      for (j = 0; j < n; j++) y[j] += (ytbl[k-1][j] - ytbl[k-2][j])*h;
    }
    if (k >= 3) {			/* ... quadratic */
      d23 = xtbl[k-2] - xtbl[k-3];
      d13 = xtbl[k-1] - xtbl[k-3];
      h = -(x - xtbl[k-1])*(x - xtbl[k-2])/(d13 * d12 * d23);
      for (j = 0; j < n; j++)
	y[j] += (-ytbl[k-3][j]*d12 + ytbl[k-2][j]*d13 - ytbl[k-1][j]*d23)*h;
    }
  }
}
